"use strict";

var strip_body = null;
var info_bar = null;
var submit_bar = null;
var grow_button = null;
var shrink_button = null;
var extra_bar = null;
//var modhash = "fb6j0tjzw312170880fbd21f842f7f1c55382dfba9bedce8da"; // XXX HACK
var modhash = null;
var getseltab = chrome.tabs.getSelected;


function enablemarvinaction(t) {
    if(t.url && t.url.indexOf("http") === 0) { 
        chrome.pageActions.enableForTab("marvin", {"tabId": t.id, "url": t.url });
    }
}

chrome.tabs.onSelectionChanged.addListener(
        function (tid) { 
            flippage(tid);
            chrome.tabs.get(tid, enablemarvinaction); 
        }
    );
chrome.tabs.onCreated.addListener(enablemarvinaction);
chrome.tabs.onUpdated.addListener( 
    function (tid, d) { 
        chrome.tabs.get(tid, enablemarvinaction); 
        if (d.url) {
            bcache[tid] = {};
            flippage(tid);
        }
    }
    );
        
chrome.pageActions.marvin.addListener( 
    function (d) { 
        bcache[d.data.tabId] = {};
        geturlinfo(d.data.tabUrl, function (inf, all, len) {
            setbcache(d.data.tabId, "url", d.data.tabUrl);
            setbcache(d.data.tabId, "url_info_all_len", len);
            setbcache(d.data.tabId, "url_info_all", all);
            setbcache(d.data.tabId, "url_info", inf);
            setbcache(d.data.tabId, "bar_enabled", true);
            showbar(bcache[d.data.tabId]);
        });
    }
    );


function init() {
    info_bar = document.getElementById("info-bar");
    submit_bar = document.getElementById("submit-bar"); 
    strip_body = document.getElementById("strip-body"); 
    grow_button = document.getElementById("grow-button");
    shrink_button = document.getElementById("shrink-button");
    extra_bar = document.getElementById("extra-bar");

    chrome.self.onConnect.addListener( function (p) {
        p.onMessage.addListener( function (msg) {
            if (msg.modhash) {
                modhash = msg.modhash;
            }
        });
    });

    gettabandflippage();
}

// Reddit API: http://code.reddit.com/browser/r2/r2/controllers/api.py
function geturlinfo(u, f) {
    var apiurl = 'http://reddit.com/api/info.json?url=';
    jreq(apiurl + encodeURIComponent(u),
        function (j) {
            var all = {};
            var l = 0; 
            var d = null;
            if (j && j.data.children.length > 0) {
                l = j.data.children.length;
                var cl = j.data.children;
                d = cl[0].data;
                for (var i in cl) {
                    cl[i].data.vote = 0;
                    if (cl[i].data.likes === true) {
                        cl[i].data.vote = 1;
                    }
                    else if (cl[i].data.likes === false) {
                        cl[i].data.vote = -1;
                    }
                    all[cl[i].data.id] = cl[i].data;
                    if (cl[i].data.score > d.score) {
                        d = cl[i].data;
                    }
                }
            }
            // 'main' post, post dict, nr posts
            f(d, all, l);
        }
    );
}

var subredditscache = null;
var subredditscache_html = "";
function compsubreddits(a, b) {
    if (a.data.url == "/r/reddit.com/") {
        return -1;
    } else if (b.data.url == "/r/reddit.com/") {
        return 1;
    } else {
        // We do case insensitive sorting unlike reddit used to, this makes it
        // easier to find things given the inconsistent 
        if (a.data.url.toLowerCase() < b.data.url.toLowerCase()) {
            return -1;
        } else if (a.data.url.toLowerCase() > b.data.url.toLowerCase()) {
            return 1;
        } else {
            return 0;
        }
    }
}
function getmysubreddits(f) {
    var apiurl = 'http://reddit.com/reddits/mine.json';
    if (subredditscache !== null) {
        f(subredditscache);
    }
    else {
        jreq(apiurl,
            function (j) {
                if (j && j.data.children.length > 0) {
                    subredditscache = j.data.children.sort(compsubreddits);
                    f(subredditscache);
                }
                else {
                    f(null);
                }
                
            }
        );
    }
}
function gensubredditselhtml(l) {
    var d,  h = "<select id='submit-subreddit'>";
    for(var i in l) {
        d = l[i].data
        h += "<option value='"+d.url+"'>"+d.url.substr(3, d.url.length-4)+"</option>";
    }
    h += "</select>"
    subredditscache_html = h;
}
getmysubreddits(gensubredditselhtml);


function rdsubmit() {
    getseltab(null, function (t) { 
        var u, i, s;
        u = "http://www.reddit.com";
        i = document.getElementById('submit-title');
        s = document.getElementById('submit-subreddit');
        if(s) {
            u += s.children[s.selectedIndex].value;
        }
        else {
            u += "/";
        }
        updatetab(u+"submit?resubmit=true&url="+encodeURIComponent(getbcache(t.id, 'url'))+"&title="+encodeURIComponent(i.value));
        hidestrip();
        i.value = '';
        bcache[t.id] = {}; 
    });
}

// down=-1  up=+1 remove-vote=0
function vote(lid, dir) {
    //var apiurl = "http://www.reddit.com/api/vote";
    var apiurl = "http://www.reddit.com/api/vote";
    var v = 0;

    if (modhash === null) {
        console.log("Modhash is missing! Need to visit reddit before you vote!");
        newtab("http://reddit.com");
        return;
    }

    getseltab(null, function (t) {
        if (dir === "up") {
            v = 1;
        }
        else if (dir === "down") {
            v = -1;
        }

        // First we undo the previous vote.
        bcache[t.id].url_info_all[lid].score += -bcache[t.id].url_info_all[lid].vote;

        // If arrow already clicked, revert vote.
        if (bcache[t.id].url_info_all[lid].vote === v) {
            v = 0;
        }
        
        // Then apply the new vote
        bcache[t.id].url_info_all[lid].score += v;

        bcache[t.id].url_info_all[lid].vote = v;

        http_post(apiurl, {"uh":modhash, "id":"t3_"+lid, "dir":v});
        updatearrows();
    });
}
// End Reddit API


// GUI Functions

// Per-tab info cache.
var bcache = {};
function setbcache(tid, k, v) {
    if (bcache[tid] === undefined) {
        bcache[tid] = {};
    }
    bcache[tid][k] = v;
}
function getbcache(tid, k) {
    if (!bcache[tid]) {
        return null;
    }
    return bcache[tid][k];
}

function flippage(tid) {
    chrome.tabs.get(tid, function (t) {
        chrome.toolstrip.collapse(); // We don't mantain the mole open on flips.
        hidestrip();
        if(getbcache(tid, "bar_enabled")) {
            showbar(bcache[tid]);
        }
    });
}

function gettabandflippage() { 
    getseltab(null, function (t) {
            flippage(t.id);
            enablemarvinaction(t);
    }); }

function updatearrows() {
    getseltab(null, function (t) {
        var u, d, e, inf;
        for (var i in bcache[t.id].url_info_all) {
            inf = bcache[t.id].url_info_all[i];
            u = 'imgs/aupgray.gif';
            d = 'imgs/adowngray.gif';
            if (inf.vote === 1) {
                u = 'imgs/aupmod.gif';
            }
            else if (inf.vote === -1) {
                d = 'imgs/adownmod.gif';
            }
            e = document.getElementById('up-'+inf.id);
            if (e) {
                e.src = u;
            }
            e = document.getElementById('down-'+inf.id);
            if (e) {
                e.src = d;
            }

            // FIXME if the score changes in number of digits the td doesn't expand.
            e = document.getElementById('score-'+inf.id);
            if (e) {
                e.innerHTML = inf.score;
            }
        }
    });
}

function genarrow(d, i) {
    return "<img onclick='vote(\""+i+"\", \""+d+"\")' id='"+d+"-"+i+"' src='imgs/a"+d+"gray.gif'>";
}

function geninfobar(j) {
    var s = '<td id="score-'+j.id+'" class="score">'+j.score+'</td>';
    s += genarrow("up", j.id);
    s += genarrow("down", j.id);
    //s += '<td><img id="down-'+j.id+'" src="imgs/adowngray.gif"></td>';
    s += '</td><td align="right">'+j.num_comments+'<img src="imgs/comments.png"></td>';
    var t = j.title.substring(0, 64); // TODO: Lenght should depend on available toolstrip space.
    s += '<td class="title">'+rdlink("/r/"+j.subreddit+"/comments/"+j.id+"/", t)+' <small>('+rdlink("/domain/"+j.domain, j.domain)+')</small>';
    s += ' by '+rdlink("/user/"+j.author, j.author)+' to <b>'+rdlink("/r/"+j.subreddit+"/", j.subreddit)+'</b></td>';
    return '<tr>'+s+'</tr>';
}
function rdlink(t, s) { return "<a href=\"javascript:newtab('http://reddit.com"+t+"');\">"+s+"</a>"; }

function hidestrip() { 
    /* strip_body.style.display = "none";*/
    submit_bar.style.display = "none";
    info_bar.style.display = "none";
    grow_button.style.display = "none";
    shrink_button.style.display = "none";
}
function showstrip() { /* strip_body.style.display = "block";*/ }
function show_info_bar(h) { 
        showstrip();
        if(h) {
            info_bar.innerHTML = h;
        }
        submit_bar.style.display = "none";
        info_bar.style.display = "inline";
        updatearrows();
}
function setupsubmitform(t) {
    var f = getbcache(t.id, 'submit_form');
    if(!f) {
        f = document.createElement("form");
        f.innerHTML = "<input id='submit-title' type='text'> <input id='submit-button' type='submit' onclick='rdsubmit()' value='Post to'> " + subredditscache_html;
        f.childNodes[0].style.width="40em";
        f.childNodes[2].style.width="5em"; // [1] is the space between the tags! Stupid HTML *yuck*
        f.style.width="70em";
        setbcache(t.id, "submit_form", f); 
    }
    if(submit_bar.childNodes.length > 0) {
        submit_bar.replaceChild(f, submit_bar.childNodes[0]);
    }
    else {
        submit_bar.appendChild(f);
    }

}

function show_submit_bar() { 
        getseltab(null, setupsubmitform);
        info_bar.style.display = "none";
        submit_bar.style.display = "inline";
}

function showbar(u) {
    grow_button.style.display = "none";
    shrink_button.style.display = "none";
    showstrip();

    if (!u || !u.url_info) {
        show_submit_bar();
    }
    else {
        show_info_bar(geninfobar(u.url_info));
        if(u.url_info_all_len > 1) {
            grow_button.style.display = "block";
        }
    }
}


function grow() {
    getseltab(null, function (t) { 
        var a = getbcache(t.id, 'url_info_all');
        if (a && getbcache(t.id, 'url_info_all_len') > 1) {
            var h = '';
            for (var i in a) {
                h += geninfobar(a[i]);
            }
            chrome.toolstrip.expand(30*getbcache(t.id, 'url_info_all_len'), '');
            show_info_bar(h);

            grow_button.style.display = "none";
            shrink_button.style.display = "block";
        }
    });
}

function shrink() {
    chrome.toolstrip.collapse();
    grow_button.style.display = "block"; // XXX We might do this when flipping to a page that can't grow!
    shrink_button.style.display = "none";
    gettabandflippage();
}

